import Interface from 'es6-interface'

import RequestHandlerInterface from './requestHandlerInterface.js'
import { resolveClassPath } from './../helpers/resolver.js'
import logger from './../helpers/logger.js'
import StateStore from './../stores/state.js'

/**
 *
 *
 *
 *
 */

class RequestHandler extends Interface(RequestHandlerInterface)
{
    /**
     *
     *
     *  @param {object} config
     *
     */
    constructor(config) {
        super()

        this.config = config

        this.stateStore = new StateStore()
        this.state = this.stateStore.findOneByName(this.config.name)
    }

    /**
     *  process actions that are saved in a source
     *
     *
     *  @param  {object} data
     *
     */
    async processActions(data)
    {
        // 
        let lastResult = false

        for (const config of this.config.actions) {

            // getting class path
            const classPath = resolveClassPath('actions', config.class)

            // import class from action
            const Action = await import(classPath)

            // create action an call run
            const action = new Action.default(this.config, config.options, data, lastResult)
            
            try {
                lastResult = await action.run()
            } catch (error) {
                logger.error(error)
            }
        }
    }
}

export default RequestHandler
